const metrics = {
  sigma: { // RMS
    option: "RMS Deviation (σ)",
    displayName: String.raw`\sigma`,
    displayBody: p => String.raw`
      \sqrt{I\left(${p}\right) - 2\vec{q}\cdot\vec{\mu}\left(${p}\right) + \left|\left|\vec{q}\right|\right|^2}
    `,
    evaluate: (data, target) => Math.sqrt(data.inertia - 2 * vectorDot(data.mu.vector, target.vector) + target.sqMag),
  },
  bigTheta: { // mean of angle
    option: "Angular Difference (Θ)",
    displayName: String.raw`\Theta`,
    displayBody: p => String.raw`
      \cos^{-1}\left( \hat{q}\cdot\vec{\nu}\left(${p}\right) \right)
    `,
    evaluate: (data, target) => rad2deg * Math.acos(vectorDot(data.nu, target.unit)),
  },
  theta: { // angle of mean
    option: "Angular Difference (θ)",
    displayName: String.raw`\theta`,
    displayBody: p => String.raw`
      \cos^{-1}\left( \hat{q}\cdot\hat{\mu}\left(${p}\right) \right)
    `,
    evaluate: (data, target) => rad2deg * Math.acos(vectorDot(data.mu.unit, target.unit)),
  },
  phi: { // hue angle
    option: "Hue Difference (ϕ)",
    displayName: String.raw`\phi`,
    displayBody: (p, space) => String.raw`
      \angle \left(\text{oproj}_{\vec{${space === "jab" ? "J" : "L"}}}{\vec{q}}, \text{oproj}_{\vec{${space === "jab" ? "J" : "L"}}}{\vec{\mu}\left(${p}\right)} \right)
    `,
    evaluate: (data, target) => angleDiff(data.mu.hue, target.hue),
  },
  delta: { // euclidean
    option: "Euclidean Distance (δ)",
    displayName: String.raw`\delta`,
    displayBody: p => String.raw`
      \left|\left| \vec{q} - \vec{\mu}\left(${p}\right) \right|\right|
    `,
    evaluate: (data, target) => vectorDist(data.mu.vector, target.vector),
  },
  ch: { // chebyshev
    option: "Chebyshev Distance (Ч)",
    displayName: "Ч",
    displayBody: p => String.raw`
      \max_{i} \left| \vec{\mu}\left(${p}\right)_i - \vec{q}_i \right|
    `,
    evaluate: (data, target) => Math.max(...data.mu.vector.map((x, i) => Math.abs(x - target.vector[i]))),
  },
  lightnessDiff: {
    option: "Lightness Difference (ℓ)",
    displayName: String.raw`\ell`,
    displayBody: (p, space) => String.raw`
      \left| \text{comp}_{\vec{${space === "jab" ? "J" : "L"}}}{\vec{q}} - \text{comp}_{\vec{${space === "jab" ? "J" : "L"}}}{\vec{\mu}\left(${p}\right)} \right|
    `,
    evaluate: (data, target) => Math.abs(data.mu.lightness - target.lightness),
  },
  inertia: {
    option: "Inertia (I)",
    displayName: "I",
    displayBody: p => String.raw`
      \frac{1}{\left|${p}\right|} \sum_{p\in ${p}}{\left|\left|\vec{p}\right|\right|^2}
    `,
    evaluate: data => data.inertia,
  },
  lightness: {
    option: "Mean Lightness (ℒ)",
    displayName: String.raw`\mathcal{L}`,
    displayBody: (p, space) => String.raw`
      \text{comp}_{\vec{${space === "jab" ? "J" : "L"}}}{\vec{\mu}\left(${p}\right)}
    `,
    evaluate: data => data.mu.lightness,
  },
  muNuAngle: {
    option: "Mu-Nu Angle (V)",
    displayName: "V",
    displayBody: p => String.raw`\angle \left( \vec{\mu}\left(${p}\right), \vec{\nu}\left(${p}\right) \right)`,
    evaluate: data => rad2deg * Math.acos(vectorDot(data.mu.unit, data.nu) / vectorMag(data.nu)),
  },
  size: {
    option: "Size (N)",
    displayName: "N",
    displayBody: p => String.raw`\left|${p}\right|`,
    evaluate: data => data.size,
  }
};

const applyMetrics = (data, target) => Object.fromEntries(
  Object.entries(metrics).map(([name, metric]) => [name, metric.evaluate(data, target)])
);
